using System;
using log4net;

namespace JamesKovacs.Common {
    public static class IoC {
        private static IDependencyContainer m_container;
        private static readonly ILog logger = LogManager.GetLogger(typeof(IoC));

        public static void Initialize(IDependencyContainer container) {
            if(m_container != null) {
                logger.Warn("IoC.Initialize() has already been called. Reinitialization outside of unit tests may indicate a problem.");
            }
            m_container = container;
        }

        public static T Resolve<T>() where T : class {
            verifyResolverInitialized();
            return m_container.Resolve<T>();
        }

        public static T Resolve<T>(string instanceName) where T : class {
            verifyResolverInitialized();
            return m_container.Resolve<T>(instanceName);
        }

        private static void verifyResolverInitialized() {
            if(m_container == null) {
                throw new InvalidOperationException("You must call IoC.Initialize() before calling resolve.");
            }
        }
    }
}