using System;
using System.Collections.Generic;

namespace JamesKovacs.Common {
    public class SimpleDependencyContainer : IDependencyContainer {
        private readonly IDictionary<Type, object> m_components = new Dictionary<Type, object>();
        private readonly IDictionary<string, object> m_namedComponents = new Dictionary<string, object>();

        public void Register<T>(T component) where T : class {
            if(m_components.ContainsKey(typeof(T))) {
                throw new InvalidOperationException(string.Format("A component has already been registered for {0}.", typeof(T).Name));
            }
            m_components.Add(typeof(T), component);
        }

        public void Register<T>(T component, string instanceName) {
            if(m_namedComponents.ContainsKey(instanceName)) {
                throw new InvalidOperationException(string.Format("A component has already been registered for {0}.", typeof(T).Name));
            }
            m_namedComponents.Add(instanceName, component);
        }

        #region IDependencyContainer Members

        public T Resolve<T>() where T : class {
            if(!m_components.ContainsKey(typeof(T))) {
                throw new InvalidOperationException(string.Format("A component has not been registered for {0}.", typeof(T).Name));
            }
            return (T)m_components[typeof(T)];
        }

        public T Resolve<T>(string instanceName) where T : class {
            if(!m_namedComponents.ContainsKey(instanceName)) {
                throw new InvalidOperationException(string.Format("A component has not been registered for {0} with the name, {1}.", typeof(T).Name, instanceName));
            }
            object namedInstance = m_namedComponents[instanceName];
            T typedNamedInstance = namedInstance as T;
            if(typedNamedInstance == null) {
                throw new InvalidOperationException(string.Format("The named instance, {0}, is of type, {1}, which is not compatible with the requested type, {2}.", instanceName, namedInstance.GetType().Name, typeof(T)));
            }
            return typedNamedInstance;
        }

        #endregion
    }
}