using System;
using System.Collections.Generic;
using JamesKovacs.Common;
using NUnit.Framework;

namespace JamesKovacs.Tests.Common {
    [TestFixture]
    public class IoCTests {
        [SetUp]
        public void Setup() {
            SimpleDependencyContainer container = new SimpleDependencyContainer();
            container.Register<IList<int>>(new List<int>());
            container.Register<IList<string>>(new List<string>(), "NamedInstance");
            IoC.Initialize(container);
        }

        [Test]
        public void CanResolveDependency() {
            Assert.IsInstanceOfType(typeof(List<int>), IoC.Resolve<IList<int>>());
        }

        [Test, ExpectedException(typeof(InvalidOperationException))]
        public void ShouldNotBeAbleToResolveUnregisteredDependency() {
            IoC.Resolve<IDisposable>();
        }

        [Test]
        public void CanResolveNamedDependency() {
            Assert.IsInstanceOfType(typeof(IList<string>), IoC.Resolve<IList<string>>("NamedInstance"));
        }
    }
}